import axios from 'axios'

const actions = {
    notify(_, message) {
        openerp.web.notification.do_warn('Atención', message)
        return false
    },
    initSale({ getters, commit, dispatch }, payload) {
        commit('setMode', payload || getters.mode)
        commit('setResult', '')
        
        commit('setLoading', true)
        commit('setCompleted', false)

        return axios.get('/eiru_sales/init').then(({data}) => {
            commit('setLoading', false)
            commit('toggleFooterButtonsVisibility')

            dispatch('explodeData', data)
        }).catch(error => {
            console.error(error)
        })
    },
    explodeData({ dispatch, commit }, payload) {
        for (let value in payload) {
            if (value === 'settings') {
                commit('updateSettings', payload[value])
                continue
            }

            dispatch(`init${value[0].toUpperCase()}${value.slice(1)}`, payload[value])
        }
    },
    createProduct({ dispatch }, payload) {
        return axios.post('createProductUrl', {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                ...payload
            }
        }).then(({data}) => {
            dispatch('receiveProduct', data.result)
        }).catch(error => {
            console.error(error)
        })
    },
    createCustomer({ dispatch }, payload) {
        return axios.post('createCustomerUrl', {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                ...payload
            }
        }).then(({data}) => {
            dispatch('receiveCustomer', data.result)
        }).catch(error => {
            console.error(error)
        })
    },
    checkCart({ getters, dispatch }) {
        return !!getters.cartItems.length || dispatch('notify', 'Necesitas agregar productos al carrito para continuar')
    },
    checkCustomer({ getters, dispatch }) {
        if (getters.processing) {
            return dispatch('notify', 'Espere mientras se está procesando')
        }

        return !!getters.selectedCustomer || dispatch('notify', 'Necesitas seleccionar un cliente para continuar')
    },
    checkPaymentMethod({ getters }) {
        if (getters.processing) {
            return dispatch('notify', 'Espere mientras se está procesando')
        }

        return true
    },
    checkAmountReceived({ getters, dispatch }) {
        if (getters.paymentType === 'cash') {
            return getters.initialPayment >= getters.amountToPay || dispatch('notify', 'El monto recibido no puede ser menor al monto a pagar')
        } else {
            return getters.initialPayment < getters.amountToPay || dispatch('notify', 'El monto recibido no puede ser igual o mayor al monto a pagar')
        }
    },
    toggleSettingsVisibility({ commit }) {
        commit('setSettingsVisibility')
    },
    changeSetting({dispatch, commit}, setting) {
        commit('setLoading', true)

        return axios.post('/eiru_sales/save_settings', {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                ...setting
            }
        }).then(({data}) => {
            dispatch('updateImages', data.result)

            commit('updateSettings', data.result)
            commit('setLoading', false)
        }).catch(error => {
            console.log(error)
        })
    },
    updateImages({ commit, getters }, data) {
        const imageType = getters.settings.imageType ? 'small' : 'big'

        if (imageType === data.imageType) {
            return
        }

        return axios.get('/eiru_sales/get_images').then(({data}) => {
            commit('setProducts', data.products)
            commit('setCustomers', data.customers)
        }).catch(error =>  {
            console.error(error)
        })
    },
    createSale({ getters, commit, dispatch }) {
        if (getters.paymentType === 'cash' && getters.initialPayment < getters.amountToPay) {
            return dispatch('notify', 'El monto recibido no puede ser menor al monto a pagar')
        } 

        if (getters.paymentType !== 'cash' && getters.initialPayment >= getters.amountToPay) {
            return dispatch('notify', 'El monto recibido no puede ser igual o mayor al monto a pagar')
        }

        commit('setLoading', true)

        return axios.post('/eiru_sales/process_sale', {
            jsonrpc: '2.0',
            method: 'call',
            params: {
                mode: getters.mode,
                items: getters.cartItems.map(item => {
                    return {
                        id: item.id,
                        quantity: item.quantity,
                        price: item.price
                    }
                }),
                total: getters.cartTotal,
                customerId: getters.selectedCustomer.id,
                paymentTermId: getters.paymentTerm.id,
                journalId: getters.selectedJournal.id,
                payment: getters.initialPayment > getters.amountToPay ? getters.amountToPay : getters.initialPayment,
                currencyId: getters.selectedCurrency.id,
                paymentMethod: getters.paymentMethod,
                bankPaymentData: {
                    bankPaymentTypeId: (getters.selectedBankPaymentType && getters.selectedBankPaymentType.id) || null,
                    chequeTypeId: (getters.selectedChequeType && getters.selectedChequeType.id) || null,
                    bankId: (getters.selectedBank && getters.selectedBank.id) || null,
                    ...getters.bankPaymentData
                }
            }
        }).then(() => {
            commit('setLoading', false)
            commit('setCompleted', true)
        }).catch(() => {
            commit('setLoading', false)
            commit('setCompleted', true)
        })
    },
    resetSettings() {
        // Ignore this
    },
    resetSale({rootState, dispatch}) {
        for (let key in rootState) {
            if (!(rootState[key] instanceof Object)) {
                continue
            }

            dispatch(`reset${key[0].toUpperCase()}${key.slice(1)}`)
        }

        dispatch('initSale')
    }
}

export default actions
